<?php
namespace Ak\Orm;

use Ak\Orm\Mapping\DiscriminatorSubscriber;
use Ak\Orm\Service\EntityManagerAwareInitializer;
use Doctrine\Common\Annotations\Reader;
use Doctrine\Common\EventManager;
use Doctrine\ORM\EntityManagerInterface;
use Doctrine\Persistence\Mapping\Driver\MappingDriver;
use Gedmo\Timestampable\TimestampableListener;
use Laminas\Form\ElementFactory;
use Laminas\ServiceManager\AbstractFactory\ReflectionBasedAbstractFactory;
use Laminas\ServiceManager\Factory\InvokableFactory;

/**
 * Pruža konfiguraciju komponente.
 *
 * @package Ak\Orm
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/orm
 */
final class ConfigProvider
{
    public function __invoke(): array
    {
        return [
            'orm' => $this->getOrmConfig(),
            'dependencies' => $this->getDependencies(),
            'filters' => $this->getFilters(),
            'form_elements' => $this->getFormElements(),
        ];
    }

    public function getOrmConfig(): array
    {
        return [
            'connection' => 'db',
            'proxy_dir' => 'var/orm/proxy',

            'cache' => [
                'name' => 'filesystem',
                'options' => [
                    'directory' => 'var/orm/metadata',
                    'extension' => '.doctrinecache.data',
                    'umask' => 0002,
                ],
            ],

            'entity_paths' => [

            ],

            'event_subscribers' => [
                Mapping\DiscriminatorSubscriber::class,
                TimestampableListener::class,
            ],
        ];
    }

    public function getDependencies(): array
    {
        return [
            'aliases' => [
                'em'                     => EntityManagerInterface::class,
                'orm.em'                 => EntityManagerInterface::class,
                'orm.annotation_reader'  => Reader::class,
                'orm.mapping_driver'     => MappingDriver::class,
                'orm.repository_factory' => Service\RepositoryFactory::class,
                'db.events' => EventManager::class,
            ],

            'factories' => [
                'orm.configuration'           => Service\ConfigurationFactory::class,
                'orm.cache'                   => Service\CacheFactory::class,
                'orm.events'                  => Service\EventManagerFactory::class,

                DiscriminatorSubscriber::class => ReflectionBasedAbstractFactory::class,
                TimestampableListener::class => InvokableFactory::class,

                Reader::class                 => Service\AnnotationReaderFactory::class,
                MappingDriver::class          => Service\MappingDriverFactory::class,
                EntityManagerInterface::class => Service\EntityManagerFactory::class,
                Service\RepositoryPluginManager::class => Service\RepositoryPluginManagerFactory::class,
                Service\RepositoryFactory::class => Service\RepositoryFactoryFactory::class,
                Middleware\EntityManagerMiddleware::class => Middleware\EntityManagerMiddlewareFactory::class,
            ],
        ];
    }

    public function getFilters(): array
    {
        return [
            'aliases' => [
                'entity' => Filter\Entity::class,
            ],

            'factories' => [
                Filter\Entity::class => InvokableFactory::class,
            ],

            'initializers' => [
                Service\EntityManagerAwareInitializer::class,
            ],
        ];
    }

    public function getFormElements(): array
    {
        return [
            'aliases' => [
                'entity_multicheckbox' => Form\Element\EntityMulticheckbox::class,
            ],

            'factories' => [
                Form\EntityForm::class => ElementFactory::class,
                Form\Element\EntityMulticheckbox::class => ElementFactory::class,
            ],

            'initializers' => [
                EntityManagerAwareInitializer::class,
            ],
        ];
    }
}
