<?php
namespace Ak\Orm\Mapping;

use Doctrine\Common\Annotations\Reader;
use Doctrine\Common\EventSubscriber;
use Doctrine\Persistence\Mapping\Driver\MappingDriver;
use Doctrine\Persistence\Mapping\MappingException;
use Doctrine\ORM\Event\LoadClassMetadataEventArgs;
use Doctrine\ORM\Events;
use ReflectionClass;
use ReflectionException;

/**
 * Class DiscriminatorListener
 *
 * @package Ak\Orm\Mapping
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/orm
 */
final class DiscriminatorSubscriber implements EventSubscriber
{
    private MappingDriver $driver;
    private Reader $reader;
    private array $map;
    private array $cachedMap = [];

    /**
     * DiscriminatorSubscriber constructor.
     *
     * @param MappingDriver $driver
     * @param Reader $reader
     */
    public function __construct(MappingDriver $driver, Reader $reader)
    {
        $this->driver = $driver;
        $this->reader = $reader;
    }

    /**
     * Returns an array of events this subscriber wants to listen to.
     *
     * @return string[]
     * @noinspection PhpMissingReturnTypeInspection
     */
    public function getSubscribedEvents()
    {
        return [Events::loadClassMetadata];
    }

    /**
     * @param LoadClassMetadataEventArgs $event
     *
     * @throws ReflectionException
     */
    public function loadClassMetadata(LoadClassMetadataEventArgs $event)
    {
        $this->map = [];
        $class = $event->getClassMetadata()->name;

        if(array_key_exists($class, $this->cachedMap)) {
            $this->overrideMetadata($event, $class);
            return;
        }

        if ($this->extractEntry($class)) {
            $this->checkFamily($class);
        } else {
            return;
        }

        $dMap = array_flip($this->map);

        foreach ($this->map as $cName => $discr) {
            $this->cachedMap[$cName]['map'] = $dMap;
            $this->cachedMap[$cName]['discr'] = $this->map[$cName];
        }

        $this->overrideMetadata($event, $class);
    }/** @noinspection PhpDocMissingThrowsInspection */

    /**
     * @param string $class
     */
    private function checkFamily(string $class): void
    {
        /** @noinspection PhpUnhandledExceptionInspection */
        $reflClass = new ReflectionClass($class);
        $parentClass = $reflClass->getParentClass();

        if ($parentClass instanceof ReflectionClass) {
            $this->checkFamily($parentClass->getName());
        } else {
            $this->cachedMap[$class]['isParent'] = true;
            /** @noinspection PhpUnhandledExceptionInspection */
            $this->checkChildren($class);
        }
    }

    /**
     * @param string $class
     *
     * @throws MappingException
     * @throws ReflectionException
     */
    private function checkChildren(string $class): void
    {
        $classNames = $this->driver->getAllClassNames();

        foreach ($classNames as $name) {
            $reflClass = new ReflectionClass($name);
            $parentClass = $reflClass->getParentClass();

            if ($parentClass instanceof ReflectionClass) {
                if ($parentClass->getName() === $class && $this->extractEntry($name)) {
                    $this->checkChildren($name);
                }
            }
        }
    }

    /**
     * @param LoadClassMetadataEventArgs $event
     * @param string $class
     */
    private function overrideMetadata(LoadClassMetadataEventArgs $event, string $class): void
    {
        $classMetadata = $event->getClassMetadata();

        $classMetadata->discriminatorMap = $this->cachedMap[$class]['map'];
        $classMetadata->discriminatorValue = $this->cachedMap[$class]['discr'];

        if (($this->cachedMap[$class]['isParent'] ?? false) === true) {
            $subclasses = $this->cachedMap[$class]['map'];
            unset($subclasses[$this->cachedMap[$class]['discr']]);

            $classMetadata->subClasses = array_values( $subclasses );
        }
    }

    /**
     * @param string $class
     * @return bool
     * @throws ReflectionException
     */
    private function extractEntry(string $class): bool
    {
        $refl = new ReflectionClass($class);
        $annotations = $this->reader->getClassAnnotations($refl);

        foreach ($annotations as $annotation) {
            if ($annotation instanceof DiscriminatorEntry) {
                $this->map[$class] = $annotation->value;
                return true;
            }
        }

        return false;
    }
}
