<?php
namespace Ak\Orm\Type;

use Ak\Orm\Hydrator\HydratedObjectHydrator;
use Doctrine\DBAL\Platforms\AbstractPlatform;
use Doctrine\DBAL\Types\Type;
use Laminas\Hydrator\HydratorInterface;
use Laminas\Json\Json;
use Throwable;

/**
 * Omogućava spremanje objekata s klasom u bazu.
 *
 * @package Ak\Orm\Type
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/orm
 */
final class HydratedObject extends Type
{
    const HYDRATED_OBJECT = 'hydrated_object';
    private ?HydratorInterface $hydrator = null;

    /**
     * Gets the SQL declaration snippet for a field of this type.
     *
     * @param array $column The field declaration.
     * @param AbstractPlatform $platform The currently used database platform.
     *
     * @return string
     * @noinspection PhpMissingReturnTypeInspection
     */
    public function getSQLDeclaration(array $column, AbstractPlatform $platform)
    {
        return $platform->getJsonTypeDeclarationSQL($column);
    }

    /**
     * Gets the name of this type.
     *
     * @return string
     * @noinspection PhpMissingReturnTypeInspection
     */
    public function getName()
    {
        return self::HYDRATED_OBJECT;
    }

    /**
     * @param mixed $value
     * @param AbstractPlatform $platform
     * @return string
     * @noinspection PhpMissingReturnTypeInspection
     */
    public function convertToDatabaseValue($value, AbstractPlatform $platform)
    {
        if (!is_object($value)) {
            return null;
        }

        $hydrator = $this->getHydrator();
        $data = $hydrator->extract($value);

        return Json::encode([
            'class' => get_class($value),
            'data' => $data,
        ]);
    }

    /**
     * @param mixed $value
     * @param AbstractPlatform $platform
     * @return mixed
     */
    public function convertToPHPValue($value, AbstractPlatform $platform)
    {
        try {
            $value = Json::decode($value, Json::TYPE_ARRAY);
        } catch (Throwable $e) {
            return null;
        }

        $object = new $value['class']();

        $hydrator = $this->getHydrator();
        $hydrator->hydrate($value['data'], $object);

        return $object;
    }

    /**
     * @return HydratorInterface
     */
    public function getHydrator(): HydratorInterface
    {
        if (null === $this->hydrator) {
            $this->hydrator = new HydratedObjectHydrator();
        }

        return $this->hydrator;
    }
}
