<?php
namespace AkTest\Orm\Form\Element;

use Ak\Orm\Form\Element\EntityMulticheckbox;
use AkTest\Orm\AbstractTestCase;
use AkTest\Orm\Entity\ConcreteEntity;
use Doctrine\DBAL\Exception as DBALException;
use Laminas\Form\FormElementManager;
use RuntimeException;

/**
 * Class EntityMulticheckboxTest
 *
 * @package AkTest\Orm\Form\Element
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/orm
 */
final class EntityMulticheckboxTest extends AbstractTestCase
{
    /**
     * @return FormElementManager
     */
    private function getFormManager(): FormElementManager
    {
        $container = $this->getContainer();
        return $container->get(FormElementManager::class);
    }

    /**
     * @param array $config
     * @return EntityMulticheckbox|object
     */
    private function getElement(array $config = []): EntityMulticheckbox
    {
        $manager = $this->getFormManager();
        return $manager->get('entity_multicheckbox', $config);
    }

    public function testIsRegistered()
    {
        $forms = $this->getFormManager();
        $this->assertTrue($forms->has('entity_multicheckbox'));

        /** @var EntityMulticheckbox $element */
        $element = $forms->get('entity_multicheckbox');

        $this->assertInstanceOf(EntityMulticheckbox::class, $element);
        $this->assertSame($this->getEntityManager(), $element->getEntityManager());
    }

    public function testWillThrowExceptionForMissingEntityClass()
    {
        $this->expectException(RuntimeException::class);

        $element = $this->getElement();
        $element->getValueOptions();
    }

    public function testWillThrowExceptionForInvalidEntityClass()
    {
        $this->expectException(RuntimeException::class);

        $element = $this->getElement([
            'options' => [
                'entity' => 'doesnt\exist',
            ],
        ]);
        $element->getValueOptions();
    }

    public function testWillThrowExceptionIfValueFieldNotSet()
    {
        $this->expectException(RuntimeException::class);

        $element = $this->getElement([
            'options' => [
                'entity' => ConcreteEntity::class,
            ],
        ]);
        $element->getValueOptions();
    }

    /**
     * @throws DBALException
     */
    public function testCanPopulateValuesWithEntities()
    {
        $conn = $this->getConnection();
        $conn->insert('concrete', ['name' => 'first']);
        $conn->insert('concrete', ['name' => 'second']);

        $element = $this->getElement([
            'options' => [
                'entity' => ConcreteEntity::class,
                'value_field' => 'name',
            ],
        ]);

        $options = $element->getValueOptions();

        $this->assertSame([
            1 => 'first',
            2 => 'second',
        ], $options);
    }

    /**
     * @throws \Doctrine\DBAL\DBALException
     */
    public function testValuesAreSorted()
    {
        $conn = $this->getConnection();
        $conn->insert('concrete', ['name' => 'zzz']);
        $conn->insert('concrete', ['name' => 'aaa']);

        $element = $this->getElement([
            'options' => [
                'entity' => ConcreteEntity::class,
                'value_field' => 'name',
            ],
        ]);

        $options = $element->getValueOptions();

        $this->assertSame([
            2 => 'aaa',
            1 => 'zzz',
        ], $options);
    }
}
